import { Filter } from "./filter.js"

function formUrlEncoded(params) {
    var data = "";
    for (var name in params) {
        if (!params.hasOwnProperty(name)) continue;
        if (data.length) data += "&";
        data += (encodeURIComponent(name) + "=" + encodeURIComponent(params[name])).replace(/%20/g, "+");
    }
    return data;
  }

class SoftpakConnector {

    constructor(t3d, baseUrl, companycode, depotcode) {
        this.baseUrl = baseUrl;
        this.depotcode = depotcode;
        this.companycode = companycode;
        this.xmlhttp = new XMLHttpRequest();
        this.t3d = t3d;
        this.stackCodes = "";
        this.filter = new Filter((filter) => { this.filterContainers(filter) });
        this.filter.addProperty('length', 'Length', "normal");
        this.filter.addProperty('customer', 'Customer', "normal");
        this.filter.addProperty('categorycode', 'Category', "normal");
        this.filter.addProperty('statuscode', 'Status', "normal");
        this.filter.addProperty('tons_tested', 'Tons Tested', "normal");
        this.filter.addProperty('cycle', 'Cycle', "normal");
        this.filter.addProperty('dangerousGoods', 'IMO', "normal");
        this.filter.addProperty('otherPort', 'Other port', "normal");
        this.filter.addProperty('out_quayjob', 'Quayjob out', "normal");
    }

    getCredentials() {
      return new Promise((resolve, reject) => {

        document.getElementById("loginButton").onclick = () => {
          
          document.getElementById("loginForm").style.display = "none";
          let username = document.getElementById("usernameinput").value;
          let password = document.getElementById("password").value;
          let credentials = {
            username:  username + "|" + this.companycode,
            password:  password
          }
          resolve(credentials);
        }
        
        document.getElementById("loginForm").style.display = "block"; 

        /* let credentials = {
          username:  'softpak' + "|" + this.companycode,
          password:  'dwgavtO.12'
        } 
        resolve(credentials); */
        
      });
    }

    async login() {
        let xmlhttp = this.xmlhttp;
        xmlhttp.open("POST", this.baseUrl + "/static/auth/j_spring_security_check", false);
        xmlhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
        xmlhttp.setRequestHeader("Accept", "application/json");
        xmlhttp.withCredentials = true;
        
        let result = await this.getCredentials();
        //console.log(result);
        
        var data = formUrlEncoded({'j_username': result.username, 'j_password': result.password});
        xmlhttp.send(data); 
        
    }
   
    getData(action) {
        let xmlhttp = new XMLHttpRequest();
        let endpoint;
        let parameterString; 

        let promise = new Promise((resolve, reject) => {

          if( action === 'containers' ) { 
            endpoint = 'getData';
            parameterString = '{"request":{"pcDepotcode": "' + this.depotcode + '"}}';
          }
          if( action === 'locations' ) { 
            endpoint = 'getStacklocations';
            parameterString = '{"request":{"pcStackCodes": "' + this.stackCodes + '"}}';
          }
          
          xmlhttp.addEventListener("load", e => {
            if (xmlhttp.readyState === xmlhttp.DONE) {
              //console.log(xmlhttp.status);
              if (xmlhttp.status === 200) {
                let myData = JSON.parse(xmlhttp.responseText);
                resolve(myData);
              }
            }            
          });
          
          xmlhttp.open("PUT", this.baseUrl + "rest/DpApp/dpVisualization/" + endpoint, true);
          xmlhttp.withCredentials = true;
          //xmlhttp.open("GET", "/src/mocked_data/" + endpoint + ".json", false);
                  
          xmlhttp.setRequestHeader("Content-Type", "application/json");
          xmlhttp.setRequestHeader("Accept", "application/json");
      
          xmlhttp.send(parameterString);
      
          // let myData = JSON.parse(xmlhttp.response);
          
          //return myData;

        });
        

        return promise;
    }

    processContainers(data) {
        let containers = data.response.dsMain.dsMain.tt_container || [];
        let heaps = data.response.dsMain.dsMain.tt_aggregate || [];
        let inoSlot = 0;
        
        // Removed containers (Containers not in response data anymore)
        let myContainers = this.t3d.containerController.containers;
        for( let i = 0; i < myContainers.length; i++) {
          let found = false;
          for( let i2 = 0; i2 < containers.length; i2++) {
             if (myContainers[i].properties.extId === containers[i2].seqnr) {
               found = true; 
             }
          }
          if (!found) {
            this.t3d.containerController.removeContainer(myContainers[i]);
          }
        }
       
        for( let i = 0; i < containers.length; i++) {
          var container = containers[i];

          let location = this.t3d.locationController.getStackLocation(container.location);
          let heapLocation = this.t3d.locationController.getHeapLocation(container.location);
          let containerObj = this.t3d.containerController.hasContainer(container.seqnr);

          if (containerObj) {
            if (!location && !heapLocation) {
              this.t3d.containerController.removeContainer(containerObj);
              containerObj = null;
            } 
          } else if (location || heapLocation) {
            if (location) { 
              containerObj = this.t3d.containerController.addContainer(container.container_length, location.getx(), location.gety(), location.getz(), location.orientation, false, {});
              containerObj.location = location;
              containerObj.location.reserved = false;
              containerObj.location.container = container;
            } else {
              //console.log('heapLocation', heapLocation);
              containerObj = this.t3d.containerController.addContainer(0, 0, 0, 0, 0, false, {});
              containerObj.location = heapLocation;
              heapLocation.containers.push(containerObj);
            }

          } else { inoSlot++; }

          if (containerObj) {
            
            // Location
            if (location && containerObj.location !== location) {
              containerObj.move(location.getx(), location.gety(), location.getz());
              containerObj.setOrientation(location.orientation);
              containerObj.location = location;s
              containerObj.location.reserved = false;
              containerObj.location.container = container;
            }

            // Properties 
            let props = containerObj.properties;
            props.containerPrefix = container.containerprefix;
            props.containerNumber = container.containernumber;
            props.extId = container.seqnr;
            props.customer = container.cust_shortname;
            props.grossWeight = container.grossweight;
            props.otherPort = container.out_portcode;
            props.length = container.container_length
            props.cycle = container.cycluscode;
            props.dangerousGoods = (container.imocode);
            props.imoclasses = container.imo_classes;
            props.out_quayjob = container.out_quayjobid + "-" + container.calc_out_vessel;
            props.categorycode = container.categorycode;
            props.tons_tested = container.tons_tested;
            props.statuscode = container.statuscode;
            props.customs = false;

            // Filters
            this.filter.addValue('length', props.length);
            this.filter.addValue('cycle', props.cycle);
            this.filter.addValue('customer', props.customer);
            this.filter.addValue('dangerousGoods', props.dangerousGoods);
            this.filter.addValue('otherPort', container.out_portcode);
            this.filter.addValue('out_quayjob', props.out_quayjob);
            this.filter.addValue('categorycode', props.categorycode);
            this.filter.addValue('statuscode', props.statuscode);
            this.filter.addValue('tons_tested', props.tons_tested);


            // Container label 
            let imoclasses = "";
            if (props.imoclasses) { imoclasses = " " + props.imoclasses;}
            let otherPort = "";
            if (props.otherPort) { otherPort = " " + props.otherPort;}
            let label = props.containerPrefix + " " + props.containerNumber + '\n' + props.length + " " + props.customer + imoclasses + otherPort + " " + (props.grossWeight / 1000).toFixed(2);
            containerObj.setLabel(label);
            
          }
           
        }

        //console.log('no slot:' + inoSlot);
        
        for( let i = 0; i < heaps.length; i++) {
          console.log('heaps', heaps);
          let heap = heaps[i];
          let location = this.t3d.locationController.getHeapLocation(heap.stacklocation);
          if (location) {
            location.setFill( heap.num_containers / location.capacity || 12) ;
            location.label = heap.num_containers.toString();
            console.log('heap: ' , heap, heap.num_containers)
          }
        }
    }

    changeLocation(containers, location, mode) {
        let endpoint;
        let body = {request: {}};
        let request = body.request; 

        request.pcContainerList = containers;
        request.pcNewLocationCode = location;
        request.pcMode = mode;
                
        this.xmlhttp.open("PUT", this.baseUrl + "/rest/DpApp/dpVisualization/changeLocation", false);
                
        this.xmlhttp.setRequestHeader("Content-Type", "application/json");
        this.xmlhttp.setRequestHeader("Accept", "application/json");
    
        this.xmlhttp.send(JSON.stringify(body));
    
        let myData = JSON.parse(this.xmlhttp.response);
        
        //console.log(myData);

        return myData;
    }

    filterContainers(filterObj) {
      //console.log(filterObj);
      this.t3d.legend.reset();

      this.t3d.locationController.heapLocations.forEach(location => {
        location.clearColors();
      })
      
      let filters = filterObj.filters;

      let containers = this.t3d.containerController.containers;
      for( let i = 0; i < containers.length; i++) {
        let container = containers[i];
        container.color = 0x3333aa;
        container.opacity = 1;
        
        for( let i2= 0; i2 < filters.length; i2++) {
          let filter = filters[i2];
          let index = filter.values.indexOf(container.properties[filter.name]);
          if(filter.filtered && index < 0 ) {
            container.color = 0xcccccc;
            container.opacity = 0.7;
          } else if(filter.colorSplit) {
            container.color = parseInt(filter.colors[index], 16);
            this.t3d.legend.add('#' + filter.colors[index], filter.values[index] );
            //console.log(container.color);

          }
        }
        //console.log('container.location.locationcode', container.location.locationCode);
        let heapLocation = this.t3d.locationController.getHeapLocation(container.location.locationCode);
        if(heapLocation) {
          //console.log('heap location:', heapLocation);
          heapLocation.addColor(container.color);
        }

        container.materialNeedsUpdate = true;
      }
      
    }

    changeLocation(containers, location, mode) {
      let endpoint;
      let body = {request: {}};
      let request = body.request; 

      request.pcContainerList = containers;
      request.pcNewLocationCode = location;
      request.pcMode = mode;
              
      this.xmlhttp.open("PUT", this.baseUrl + "/rest/DpApp/dpVisualization/changeLocation", false);
              
      this.xmlhttp.setRequestHeader("Content-Type", "application/json");
      this.xmlhttp.setRequestHeader("Accept", "application/json");
  
      this.xmlhttp.send(JSON.stringify(body));
  
      let myData = JSON.parse(this.xmlhttp.response);
      
      console.log(myData);

      return myData;
  }

}

export { SoftpakConnector };